@@ -19,6 +19,7 @@ import collections
 import copy
 import json
 import os
+import re
 import sys
 import warnings
 
@@ -105,9 +106,12 @@ def _get_os_environ(envvar_prefix=None):
     for k in environkeys:
         newkey = k.split('_', 1)[-1].lower()
         ret[newkey] = os.environ[k]
-    # If the only environ keys are cloud and region_name, don't return anything
-    # because they are cloud selectors
-    if set(environkeys) - set(['OS_CLOUD', 'OS_REGION_NAME']):
+    # If the only environ keys are selectors or behavior modification, don't
+    # return anything
+    selectors = set([
+        'OS_CLOUD', 'OS_REGION_NAME',
+        'OS_CLIENT_CONFIG_FILE', 'OS_CLIENT_SECURE_FILE', 'OS_CLOUD_NAME'])
+    if set(environkeys) - selectors:
         return ret
     return None
 
@@ -146,7 +150,9 @@ def _fix_argv(argv):
     # over the place.
     processed = collections.defaultdict(list)
     for index in range(0, len(argv)):
-        if argv[index].startswith('--'):
+        # If the value starts with '--' and has '-' or '_' in it, then
+        # it's worth looking at it
+        if re.match('^--.*(_|-)+.*', argv[index]):
             split_args = argv[index].split('=')
             orig = split_args[0]
             new = orig.replace('_', '-')
@@ -173,19 +179,28 @@ class OpenStackConfig(object):
     def __init__(self, config_files=None, vendor_files=None,
                  override_defaults=None, force_ipv4=None,
                  envvar_prefix=None, secure_files=None,
-                 pw_func=None, session_constructor=None):
+                 pw_func=None, session_constructor=None,
+                 app_name=None, app_version=None,
+                 load_yaml_config=True):
         self.log = _log.setup_logging(__name__)
         self._session_constructor = session_constructor
+        self._app_name = app_name
+        self._app_version = app_version
 
-        self._config_files = config_files or CONFIG_FILES
-        self._secure_files = secure_files or SECURE_FILES
-        self._vendor_files = vendor_files or VENDOR_FILES
+        if load_yaml_config:
+            self._config_files = config_files or CONFIG_FILES
+            self._secure_files = secure_files or SECURE_FILES
+            self._vendor_files = vendor_files or VENDOR_FILES
+        else:
+            self._config_files = []
+            self._secure_files = []
+            self._vendor_files = []
 
-        config_file_override = os.environ.pop('OS_CLIENT_CONFIG_FILE', None)
+        config_file_override = os.environ.get('OS_CLIENT_CONFIG_FILE')
         if config_file_override:
             self._config_files.insert(0, config_file_override)
 
-        secure_file_override = os.environ.pop('OS_CLIENT_SECURE_FILE', None)
+        secure_file_override = os.environ.get('OS_CLIENT_SECURE_FILE')
         if secure_file_override:
             self._secure_files.insert(0, secure_file_override)
 
@@ -214,12 +229,12 @@ class OpenStackConfig(object):
         else:
             # Get the backwards compat value
             prefer_ipv6 = get_boolean(
-                os.environ.pop(
+                os.environ.get(
                     'OS_PREFER_IPV6', client_config.get(
                         'prefer_ipv6', client_config.get(
                             'prefer-ipv6', True))))
             force_ipv4 = get_boolean(
-                os.environ.pop(
+                os.environ.get(
                     'OS_FORCE_IPV4', client_config.get(
                         'force_ipv4', client_config.get(
                             'broken-ipv6', False))))
@@ -231,7 +246,7 @@ class OpenStackConfig(object):
                 self.force_ipv4 = True
 
         # Next, process environment variables and add them to the mix
-        self.envvar_key = os.environ.pop('OS_CLOUD_NAME', 'envvars')
+        self.envvar_key = os.environ.get('OS_CLOUD_NAME', 'envvars')
         if self.envvar_key in self.cloud_config['clouds']:
             raise exceptions.OpenStackConfigException(
                 '"{0}" defines a cloud named "{1}", but'
@@ -239,9 +254,8 @@ class OpenStackConfig(object):
                 ' either your environment based cloud, or one of your'
                 ' file-based clouds.'.format(self.config_filename,
                                              self.envvar_key))
-        # Pull out OS_CLOUD so that if it's the only thing set, do not
-        # make an envvars cloud
-        self.default_cloud = os.environ.pop('OS_CLOUD', None)
+
+        self.default_cloud = os.environ.get('OS_CLOUD')
 
         envvars = _get_os_environ(envvar_prefix=envvar_prefix)
         if envvars:
@@ -647,6 +661,7 @@ class OpenStackConfig(object):
         the keystoneauth Auth Plugin Options and os-cloud. Also, peek in the
         argv to see if all of the auth plugin options should be registered
         or merely the ones already configured.
+
         :param argparse.ArgumentParser: parser to attach argparse options to
         :param list argv: the arguments provided to the application
         :param string service_keys: Service or list of services this argparse
@@ -854,61 +869,13 @@ class OpenStackConfig(object):
             # Set to notused rather than None because validate_auth will
             # strip the value if it's actually python None
             config['auth']['token'] = 'notused'
+        elif config['auth_type'] == 'token_endpoint':
+            # Humans have been trained to use a thing called token_endpoint
+            # That it does not exist in keystoneauth is irrelvant- it not
+            # doing what they want causes them sorrow.
+            config['auth_type'] = 'admin_token'
         return loading.get_plugin_loader(config['auth_type'])
 
-    def _validate_auth_ksc(self, config, cloud):
-        try:
-            import keystoneclient.auth as ksc_auth
-        except ImportError:
-            return config
-
-        # May throw a keystoneclient.exceptions.NoMatchingPlugin
-        plugin_options = ksc_auth.get_plugin_class(
-            config['auth_type']).get_options()
-
-        for p_opt in plugin_options:
-            # if it's in config.auth, win, kill it from config dict
-            # if it's in config and not in config.auth, move it
-            # deprecated loses to current
-            # provided beats default, deprecated or not
-            winning_value = self._find_winning_auth_value(
-                p_opt,
-                config['auth'],
-            )
-            if not winning_value:
-                winning_value = self._find_winning_auth_value(
-                    p_opt,
-                    config,
-                )
-
-            # if the plugin tells us that this value is required
-            # then error if it's doesn't exist now
-            if not winning_value and p_opt.required:
-                raise exceptions.OpenStackConfigException(
-                    'Unable to find auth information for cloud'
-                    ' {cloud} in config files {files}'
-                    ' or environment variables. Missing value {auth_key}'
-                    ' required for auth plugin {plugin}'.format(
-                        cloud=cloud, files=','.join(self._config_files),
-                        auth_key=p_opt.name, plugin=config.get('auth_type')))
-
-            # Clean up after ourselves
-            for opt in [p_opt.name] + [o.name for o in p_opt.deprecated_opts]:
-                opt = opt.replace('-', '_')
-                config.pop(opt, None)
-                config['auth'].pop(opt, None)
-
-            if winning_value:
-                # Prefer the plugin configuration dest value if the value's key
-                # is marked as depreciated.
-                if p_opt.dest is None:
-                    config['auth'][p_opt.name.replace('-', '_')] = (
-                        winning_value)
-                else:
-                    config['auth'][p_opt.dest] = winning_value
-
-        return config
-
     def _validate_auth(self, config, loader):
         # May throw a keystoneauth1.exceptions.NoMatchingPlugin
 
@@ -935,6 +902,15 @@ class OpenStackConfig(object):
                 winning_value,
             )
 
+            if winning_value:
+                # Prefer the plugin configuration dest value if the value's key
+                # is marked as deprecated.
+                if p_opt.dest is None:
+                    good_name = p_opt.name.replace('-', '_')
+                    config['auth'][good_name] = winning_value
+                else:
+                    config['auth'][p_opt.dest] = winning_value
+
             # See if this needs a prompting
             config = self.option_prompt(config, p_opt)
 
@@ -1085,6 +1061,7 @@ class OpenStackConfig(object):
                         config[key] = val
 
         config = self.magic_fixes(config)
+        config = self._normalize_keys(config)
 
         # NOTE(dtroyer): OSC needs a hook into the auth args before the
         #                plugin is loaded in order to maintain backward-
@@ -1092,21 +1069,9 @@ class OpenStackConfig(object):
         config = self.auth_config_hook(config)
 
         if validate:
-            try:
-                loader = self._get_auth_loader(config)
-                config = self._validate_auth(config, loader)
-                auth_plugin = loader.load_from_options(**config['auth'])
-            except Exception as e:
-                # We WANT the ksa exception normally
-                # but OSC can't handle it right now, so we try deferring
-                # to ksc. If that ALSO fails, it means there is likely
-                # a deeper issue, so we assume the ksa error was correct
-                self.log.debug("Deferring keystone exception: {e}".format(e=e))
-                auth_plugin = None
-                try:
-                    config = self._validate_auth_ksc(config, cloud)
-                except Exception:
-                    raise e
+            loader = self._get_auth_loader(config)
+            config = self._validate_auth(config, loader)
+            auth_plugin = loader.load_from_options(**config['auth'])
         else:
             auth_plugin = None
 
@@ -1127,11 +1092,13 @@ class OpenStackConfig(object):
         return cloud_config.CloudConfig(
             name=cloud_name,
             region=config['region_name'],
-            config=self._normalize_keys(config),
+            config=config,
             force_ipv4=force_ipv4,
             auth_plugin=auth_plugin,
             openstack_config=self,
             session_constructor=self._session_constructor,
+            app_name=self._app_name,
+            app_version=self._app_version,
         )
 
     def get_one_cloud_osc(
