@@ -16,6 +16,7 @@ import argparse
 import copy
 import os
 
+import extras
 import fixtures
 import testtools
 import yaml
@@ -226,7 +227,7 @@ class TestConfig(base.TestCase):
         c = config.OpenStackConfig(config_files=['nonexistent'],
                                    vendor_files=['nonexistent'],
                                    secure_files=[self.secure_yaml])
-        cc = c.get_one_cloud(cloud='_test_cloud_no_vendor')
+        cc = c.get_one_cloud(cloud='_test_cloud_no_vendor', validate=False)
         self.assertEqual('testpass', cc.auth['password'])
 
     def test_get_cloud_names(self):
@@ -384,7 +385,7 @@ class TestConfigArgparse(base.TestCase):
                                    vendor_files=[self.vendor_yaml])
 
         cc = c.get_one_cloud(
-            cloud='_test_cloud_regions', argparse=self.options)
+            cloud='_test_cloud_regions', argparse=self.options, validate=False)
         self.assertEqual(cc.region_name, 'region2')
         self.assertEqual(cc.snack_type, 'cookie')
 
@@ -481,7 +482,7 @@ class TestConfigArgparse(base.TestCase):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
                                    vendor_files=[self.vendor_yaml])
 
-        cc = c.get_one_cloud(argparse=self.options)
+        cc = c.get_one_cloud(argparse=self.options, validate=False)
         self.assertIsNone(cc.cloud)
         self.assertEqual(cc.region_name, 'region2')
         self.assertEqual(cc.snack_type, 'cookie')
@@ -490,7 +491,7 @@ class TestConfigArgparse(base.TestCase):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
                                    vendor_files=[self.vendor_yaml])
 
-        cc = c.get_one_cloud(**self.args)
+        cc = c.get_one_cloud(validate=False, **self.args)
         self.assertIsNone(cc.cloud)
         self.assertEqual(cc.region_name, 'region2')
         self.assertEqual(cc.snack_type, 'cookie')
@@ -577,6 +578,37 @@ class TestConfigArgparse(base.TestCase):
         self.assertEqual(cc.region_name, 'region2')
         self.assertEqual('my-network', cc.config['external_network'])
 
+    def test_get_one_cloud_no_yaml_no_cloud(self):
+        c = config.OpenStackConfig(load_yaml_config=False)
+
+        self.assertRaises(
+            exceptions.OpenStackConfigException,
+            c.get_one_cloud,
+            cloud='_test_cloud_regions', region_name='region2', argparse=None)
+
+    def test_get_one_cloud_no_yaml(self):
+        c = config.OpenStackConfig(load_yaml_config=False)
+
+        cc = c.get_one_cloud(
+            region_name='region2', argparse=None,
+            **base.USER_CONF['clouds']['_test_cloud_regions'])
+        # Not using assert_cloud_details because of cache settings which
+        # are not present without the file
+        self.assertIsInstance(cc, cloud_config.CloudConfig)
+        self.assertTrue(extras.safe_hasattr(cc, 'auth'))
+        self.assertIsInstance(cc.auth, dict)
+        self.assertIsNone(cc.cloud)
+        self.assertIn('username', cc.auth)
+        self.assertEqual('testuser', cc.auth['username'])
+        self.assertEqual('testpass', cc.auth['password'])
+        self.assertFalse(cc.config['image_api_use_tasks'])
+        self.assertTrue('project_name' in cc.auth or 'project_id' in cc.auth)
+        if 'project_name' in cc.auth:
+            self.assertEqual('testproject', cc.auth['project_name'])
+        elif 'project_id' in cc.auth:
+            self.assertEqual('testproject', cc.auth['project_id'])
+        self.assertEqual(cc.region_name, 'region2')
+
     def test_fix_env_args(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
                                    vendor_files=[self.vendor_yaml])
@@ -622,7 +654,7 @@ class TestConfigArgparse(base.TestCase):
                                    vendor_files=[self.vendor_yaml])
 
         cc = c.get_one_cloud(
-            cloud='envvars', argparse=self.options)
+            cloud='envvars', argparse=self.options, validate=False)
         self.assertEqual(cc.auth['project_name'], 'project')
 
     def test_argparse_default_no_token(self):
@@ -650,7 +682,7 @@ class TestConfigArgparse(base.TestCase):
         opts, _remain = parser.parse_known_args(
             ['--os-auth-token', 'very-bad-things',
              '--os-auth-type', 'token'])
-        cc = c.get_one_cloud(argparse=opts)
+        cc = c.get_one_cloud(argparse=opts, validate=False)
         self.assertEqual(cc.config['auth_type'], 'token')
         self.assertEqual(cc.config['auth']['token'], 'very-bad-things')
 
@@ -670,6 +702,17 @@ class TestConfigArgparse(base.TestCase):
         self.assertEqual(cc.config['auth']['password'], 'pass')
         self.assertEqual(cc.config['auth']['auth_url'], 'auth-url')
 
+    def test_argparse_action_append_no_underscore(self):
+        c = config.OpenStackConfig(config_files=[self.no_yaml],
+                                   vendor_files=[self.no_yaml],
+                                   secure_files=[self.no_yaml])
+        parser = argparse.ArgumentParser()
+        parser.add_argument('--foo', action='append')
+        argv = ['--foo', '1', '--foo', '2']
+        c.register_argparse_arguments(parser, argv=argv)
+        opts, _remain = parser.parse_known_args(argv)
+        self.assertEqual(opts.foo, ['1', '2'])
+
     def test_argparse_underscores_duplicate(self):
         c = config.OpenStackConfig(config_files=[self.no_yaml],
                                    vendor_files=[self.no_yaml],
